SystemOrganization addCategory: #'ConfigurationCommandLineHandler-Core'!

CommandLineHandler subclass: #ConfigurationCommandLineHandler
	instanceVariableNames: 'version repositoryURL configurationName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ConfigurationCommandLineHandler-Core'!

!ConfigurationCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: config [--help] <repository url> [<configuration>] [--install[=<version>]] [--groups=<group>,<group>...] [--username=<username>] [--password=<password>]
	--help              show this help message
	<repository url>    A Monticello repository name 
	<configuration>     A valid Metacello Configuration name
	<version>           A valid version for the given configuration
	<group>             A valid Metacello group name
	<username>          An optional username to access the configuration's repository
	<password>          An optional password to access the configuration's repository
	
Examples:
	# display this help message
	$PharoVM My.image config
	
	# list all configurations of a repository
	$PharoVM My.image config $MC_REPOS_URL
	
	# list all the available versions of a confgurtation
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo
	
	# install the stable version
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install
	
	#install a specific version '1.5'
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install=1.5
	
	#install a specific version '1.5' and only a specific group 'Tests'
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install=1.5 --groups=Tests,"Group w. spaces"
!

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
commandName
	^ 'config'! !

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
description
	^ 'Install and inspect Metacello Configurations from the command line'! !

!ConfigurationCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 1/12/2013 20:08'!
activate
	self activateHelp ifTrue: [ ^ self ].
	
	(self hasOption: 'install')
		ifFalse: [ ^ self list ].
		
	self installConfiguration.
		! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:55'!
configuration
	self gofer
		package: self configurationName;
		load.
	^ Smalltalk globals at: self configurationName asSymbol! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:54'!
configurationName
	^ configurationName ifNil: [ configurationName := self argumentAt: 2 ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:46'!
errorNoConfigurationsFound

	self exitFailure: 'No Configurations found in ', repositoryURL asString! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/22/2013 23:40'!
gofer
	| username password |
	username := (self hasOption: 'username')
		ifTrue: [ self optionAt: 'username' ]
		ifFalse: [ String new ].
	password := (self hasOption: 'password')
		ifTrue: [ self optionAt: 'password' ]
		ifFalse: [ String new ].
	^ Gofer new
		url: self repositoryURL username: username password: password;
		yourself! !

!ConfigurationCommandLineHandler methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 13:20'!
hasConfiguration
	self flag: 'TODO: should use proper cli parser Coral??'.
	^ (self arguments size > 1) and: [
		((self argumentAt: 2) beginsWith: '-') not ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:12'!
installConfiguration
	self installVersion: ((self optionAt: 'install') ifNil: [ #stable ])! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'SeanDeNigris 3/8/2013 10:56'!
installVersion: aVersionName
	| metacelloVersion |
	self inform: 'Installing ', self configurationName, ' ', aVersionName.
	
	metacelloVersion := (self metacelloVersion: aVersionName).
	
	(self hasOption: 'groups')
		ifTrue: [ | groupString groups |
			groupString := (self optionAt: 'groups').
			groups := groupString splitOn: $,.
			metacelloVersion load: groups ]
		ifFalse: [ metacelloVersion load ].
	
	Smalltalk snapshot: true andQuit: true.! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:07'!
list
	self hasConfiguration 
		ifTrue: [ self listConfigurationDetails ]
		ifFalse: [ self listConfigurations ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:02'!
listConfigurationDetails
	"List possible configurations of the given repository"
	
	self printMetacelloProjectDetails: self project.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:42'!
listConfigurations
	"List possible configurations of the given repository"
	
	| configurations |
	
	configurations := self loadConfigurationNames.
	configurations ifEmpty: [ ^ self errorNoConfigurationsFound ].
	self printConfigurations: configurations.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:53'!
loadConfigurationNames
	| references configurations |
	
	references := self gofer allResolved.
		
	references := references 
			collect: #packageName 
			thenSelect: [ :name| name beginsWith: 'ConfigurationOf' ].
	
	^ references asSet asSortedCollection ! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:25'!
loadRepositoryURL
	^ self argumentAt: 1.! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/25/2012 19:24'!
metacelloVersion: aVersionName
	| project |
	project := self project.
	^ [ project version: aVersionName ] 
		on: MetacelloVersionDoesNotExistError do: [ :error |
			aVersionName = 'last'
				ifTrue: [
					"manual fallback since there is no symbolic name for lastVersion"
					project lastVersion ]
				ifFalse: [ 
					"symbols and strings are not equal in Meteacello..."
					project version: aVersionName asSymbol ]].! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 11/2/2012 15:26'!
printConfigurations: configurations

	self inform: (String streamContents:  [ :s|
		s
			nextPutAll: 'Found ';
			print: configurations size;
			nextPutAll: ' Configuration';
			nextPutAll: (configurations size = 1 ifTrue: ':' ifFalse: 's:') ]).
	
	configurations do: [ :name | 
			self << name. 
			self stdout lf ].
	! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 13:59'!
printMetacelloProjectDetails: metacelloProject
	self inform: 'Available versions for ', self configurationName.
	
	metacelloProject symbolicVersionMap keysDo: [ :key|
		self stdout print: key; lf ].
	
	metacelloProject map keysDo: [ :key|
		self stdout nextPutAll: key; lf ]! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 14:02'!
project
	^ self configuration project! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:24'!
repositoryURL
	^ repositoryURL ifNil: [ repositoryURL := self loadRepositoryURL ]! !
